(*----------------------------------------------------------------------------*
 *  Direct3D VCLfullscreenD3D9                                                *
 *  shows how to use Delphi VCL dialogs in Direct3D 9.0 fullscreen mode       *
 *                                                                            *
 *  Note: DirectX 9.0 Summer 2003 SDK compatible frawework required           *
 *                                                                            *
 *  Copyright (c) 2004 by Alexey Barkovoy                                     *
 *  E-Mail: clootie@ixbt.com                                                  *
 *                                                                            *
 *  Modified: 26-Jan-2004                                                     *
 *                                                                            *
 *  Latest version can be downloaded from:                                    *
 *     http://clootie.narod.ru/delphi/download_custom.html                    *
 *----------------------------------------------------------------------------*)
// Revision 1.1 -- modified for Summer2003 SDK -
//                 removed BuildPresentParamsFromSettings procedure

unit D3Dpart;

interface

uses
  Windows, Forms, Direct3D9, D3DX9,
  D3DApp, D3DFont, D3DUtil, DXUtil, D3DFile;

type
  CMyD3DApplication = class(CD3DApplication)
    m_pFont: CD3DFont;        // Font for drawing text
    m_ArcBall: CD3DArcBall;   // Mouse rotation utility

    fTeapot: ID3DXMesh;

    m_matArcballWorld: TD3DXMatrix;
  protected
    function OneTimeSceneInit: HRESULT; override;
    function AdjustWindowForChange: HRESULT; override;
    function InitDeviceObjects: HRESULT; override;
    function RestoreDeviceObjects: HRESULT; override;
    function InvalidateDeviceObjects: HRESULT; override;
    function DeleteDeviceObjects: HRESULT; override;
    function Render: HRESULT; override;
    function FrameMove: HRESULT; override;
    function ConfirmDevice(const pCaps: TD3DCaps9; dwBehavior: DWORD;
      adapterFormat: TD3DFormat; backBufferFormat: TD3DFormat): HRESULT; override;
    function FinalCleanup: HRESULT; override;
  public
    fAppForm: TForm;
    m_hwndRenderWindow:         HWND;
    m_hwndRenderFullScreen:     HWND;
    constructor Create;
    procedure AssignWindowsHandles;
    function MsgProc(hWnd: HWND; uMsg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT; override;
  end;

implementation


//-----------------------------------------------------------------------------
// Name: CMyD3DApplication()
// Desc: Application constructor. Sets attributes for the app.
//-----------------------------------------------------------------------------
constructor CMyD3DApplication.Create;
begin
  inherited;
  m_strWindowTitle := 'Direct3D Display Panel';
  m_bAllowDialogBoxMode := True;
  m_bShowCursorWhenFullscreen := True;
  m_d3dEnumeration.AppUsesDepthBuffer := True;

  m_pFont := CD3DFont.Create('Arial', 12, D3DFONT_BOLD);
  m_ArcBall := CD3DArcBall.Create;
  m_ArcBall.SetTranslationRadius(10);
end;


//-----------------------------------------------------------------------------
// Name: OneTimeSceneInit()
// Desc: Called during initial app startup, this function performs all the
//       permanent initialization.
//-----------------------------------------------------------------------------
function CMyD3DApplication.OneTimeSceneInit: HRESULT;
begin
  // Set cursor to indicate that user can move the object with the mouse
{$IFDEF WIN64}
  SetClassLongPtr(m_hWnd, GCLP_HCURSOR, LONG_PTR(LoadCursor(0, IDC_SIZEALL)));
{$ELSE}
  SetClassLong(m_hWnd, GCL_HCURSOR, LoadCursor(0, IDC_SIZEALL));
{$ENDIF}

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: FrameMove()
// Desc: Called once per frame, the call is the entry point for animating
//       the scene.
//-----------------------------------------------------------------------------
function CMyD3DApplication.FrameMove: HRESULT;
begin
  // Update rotation matrix from ArcBall
  m_matArcballWorld:= m_ArcBall.GetRotationMatrix^;
  D3DXMatrixMultiply(m_matArcballWorld, m_matArcballWorld, m_ArcBall.GetTranslationMatrix^);
  m_pd3dDevice.SetTransform(D3DTS_WORLD, m_matArcballWorld);

  Result:= S_OK;
end;



//-----------------------------------------------------------------------------
// Name: Render()
// Desc: Called once per frame, the call is the entry point for 3d
//       rendering. This function sets up render states, clears the
//       viewport, and renders the scene.
//-----------------------------------------------------------------------------
function CMyD3DApplication.Render: HRESULT;
begin
  // Clear the viewport
  m_pd3dDevice.Clear(0, nil, D3DCLEAR_TARGET or D3DCLEAR_ZBUFFER,
                     D3DCOLOR_RGBA(0,0,0,0), 1.0, 0);

  // Begin the scene
  if SUCCEEDED(m_pd3dDevice.BeginScene) then
  begin
    fTeapot.DrawSubset(0);

    // Output statistics
    m_pFont.DrawText(2,  0, D3DCOLOR_ARGB(255,255,255,0), m_strFrameStats);
    m_pFont.DrawText(2, 20, D3DCOLOR_ARGB(255,255,255,0), m_strDeviceStats);

    m_pFont.DrawText(2, 60, D3DCOLOR_ARGB(255,255,255,0), 'Press Space to show dialog');
    // End the scene
    m_pd3dDevice.EndScene;
  end;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: AdjustWindowForChange()
// Desc: Adjusts the window properties for windowed or fullscreen mode
//-----------------------------------------------------------------------------
function CMyD3DApplication.AdjustWindowForChange: HRESULT;
begin
  if (m_bWindowed) then
  begin
    ShowWindow(m_hwndRenderFullScreen, SW_HIDE);
    m_hWnd:= m_hwndRenderWindow;
  end else
  begin
    if IsIconic(m_hwndRenderFullScreen) then
      ShowWindow(m_hwndRenderFullScreen, SW_RESTORE);

    ShowWindow(m_hwndRenderFullScreen, SW_SHOW);
    m_hWnd:= m_hwndRenderFullScreen;
  end;
  Result:= S_OK;
end;

//-----------------------------------------------------------------------------
// Name: InitDeviceObjects()
// Desc: Initialize scene objects.
//-----------------------------------------------------------------------------
function CMyD3DApplication.InitDeviceObjects:HRESULT;
begin
  // Initialize the font's internal textures
  m_pFont.InitDeviceObjects(m_pd3dDevice);

  m_pd3dDevice.SetDialogBoxMode(True);

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: RestoreDeviceObjects()
// Desc: Initialize scene objects.
//-----------------------------------------------------------------------------
function CMyD3DApplication.RestoreDeviceObjects: HRESULT;
var
  fAspect: Single;
  m_matProjection: TD3DXMatrix;
  matView: TD3DXMatrix;
  vEyePt, vLookatPt, vUpVec: TD3DXVector3;
  Light: TD3DLight9;
  Mtrl: TD3DMaterial9;
begin
  m_pFont.RestoreDeviceObjects;
  D3DXCreateTeapot(m_pd3dDevice, fTeapot, nil);
  m_ArcBall.SetWindow(m_d3dsdBackBuffer.Width, m_d3dsdBackBuffer.Height);

  // Update projection matrix based on window dimensions
  fAspect := m_d3dsdBackBuffer.Width / m_d3dsdBackBuffer.Height;
  D3DXMatrixPerspectiveFovLH(m_matProjection, D3DX_PI / 4, fAspect, 1.0, 1000.0);
  m_pd3dDevice.SetTransform(D3DTS_PROJECTION, m_matProjection);

  vEyePt    := D3DXVector3(0.0, 5.0, 1.0);
  vLookatPt := D3DXVector3(0.0, 0.0, 0.0);
  vUpVec    := D3DXVector3(0.0, 1.0, 0.0);
  D3DXMatrixLookAtLH(matView, vEyePt, vLookatPt, vUpVec);
  m_pd3dDevice.SetTransform(D3DTS_VIEW, matView);

  D3DUtil_InitLight(Light, D3DLIGHT_DIRECTIONAL, 1, -1, 1);
  Light.Diffuse:= D3DXColor(0, 1, 0.5, 0);
  m_pd3dDevice.SetLight(0, Light);
  m_pd3dDevice.LightEnable(0, True);

  D3DUtil_InitMaterial(Mtrl, 1, 0, 1, 1);
  m_pd3dDevice.SetMaterial(Mtrl);

  m_pd3dDevice.SetRenderState(D3DRS_AMBIENT, $60606060);

  m_pd3dDevice.SetRenderState(D3DRS_LIGHTING, iTrue);
  m_pd3dDevice.SetRenderState(D3DRS_ZENABLE, iTrue);

  m_pd3dDevice.SetTextureStageState(0, D3DTSS_COLOROP,   D3DTOP_MODULATE);
  m_pd3dDevice.SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
  m_pd3dDevice.SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_DIFFUSE);
  m_pd3dDevice.SetTextureStageState(0, D3DTSS_ALPHAOP,   D3DTOP_DISABLE);

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: InvalidateDeviceObjects()
// Desc:
//-----------------------------------------------------------------------------
function CMyD3DApplication.InvalidateDeviceObjects: HRESULT;
begin
  m_pFont.InvalidateDeviceObjects;
  fTeapot:= nil;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: DeleteDeviceObjects()
// Desc: Called when the app is exiting, or the device is being changed,
//       this function deletes any device dependent objects.
//-----------------------------------------------------------------------------
function CMyD3DApplication.DeleteDeviceObjects: HRESULT;
begin
  m_pFont.DeleteDeviceObjects;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: ConfirmDevice()
// Desc: Called during device initialization, this code checks the device
//       for some minimum set of capabilities
//-----------------------------------------------------------------------------
function CMyD3DApplication.ConfirmDevice(const pCaps: TD3DCaps9; dwBehavior: DWORD;
  adapterFormat: TD3DFormat; backBufferFormat: TD3DFormat): HRESULT;
begin
  Result:= E_FAIL;

  // Need to support post-pixel processing (for alpha blending)
  if FAILED(m_pD3D.CheckDeviceFormat(pCaps.AdapterOrdinal, pCaps.DeviceType,
       adapterFormat, D3DUSAGE_RENDERTARGET or D3DUSAGE_QUERY_POSTPIXELSHADER_BLENDING,
       D3DRTYPE_SURFACE, backBufferFormat))
  then Exit;

  // Only D3DFMT_X1R5G5B5, D3DFMT_R5G6B5, or D3DFMT_X8R8G8B8 format
  // are supported for fullscreen GDI mode.
  if not (backBufferFormat in [D3DFMT_X1R5G5B5, D3DFMT_R5G6B5, D3DFMT_X8R8G8B8]) then Exit;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: FinalCleanup()
// Desc: Called before the app exits, this function gives the app the chance
//       to cleanup after itself.
//-----------------------------------------------------------------------------
function CMyD3DApplication.FinalCleanup: HRESULT;
begin
  SAFE_DELETE(m_pFont);

  Result:= S_OK;
end;

//-----------------------------------------------------------------------------
// Name: MsgProc()
// Desc: Message proc function to handle user input
//-----------------------------------------------------------------------------
function CMyD3DApplication.MsgProc(hWnd: HWND; uMsg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT;
begin
  // Pass mouse messages to the ArcBall so it can build internal matrices
  m_ArcBall.HandleMessages(hWnd, uMsg, wParam, lParam);
  Result:= inherited MsgProc(hWnd, uMsg, wParam, lParam);
end;

procedure CMyD3DApplication.AssignWindowsHandles;
begin
  // Note that for the VCL samples, the device window and focus window
  // are not the same.
  m_hWnd := m_hwndRenderWindow;
  m_hWndFocus := m_hwndRenderFullScreen;
end;

end.

